@@ -24,6 +24,7 @@
     limitations under the License.
 """
 import fnmatch
+import sys
 
 from PyQt5.QtCore import pyqtProperty, QRect
 from PyQt5.QtWidgets import QWidget, QLabel, QHBoxLayout
@@ -47,14 +48,22 @@ def match_extension(name, extensions=None):
     return True in state
 
 
+def root_url():
+    if sys.platform.startswith('win'):
+        return u"c://"
+    else:
+        return u"/"
+
+
 class WidgetImageText(QWidget):
     # inspired by
     # https://stackoverflow.com/questions/45896291/how-to-show-image-and-text-at-same-cell-in-qtablewidget-in-pyqt
     # slightly modified
-    def __init__(self, text, img, parent=None):
+    def __init__(self, text, img, value, parent=None):
         QWidget.__init__(self, parent)
         self._text = text
         self._img = img
+        self._value = value
         self.setLayout(QHBoxLayout())
         self.lbPixmap = QLabel(self)
         self.lbText = QLabel(self)
@@ -72,20 +81,11 @@ class WidgetImageText(QWidget):
     def img(self):
         return self._img
 
-    @img.setter
-    def total(self, value):
-        if self._img == value:
-            return
-        self._img = value
-        self.initUi()
-
     @pyqtProperty(str)
     def text(self):
         return self._text
 
-    @text.setter
-    def text(self, value):
-        if self._text == value:
-            return
-        self._text = value
-        self.initUi()
\ No newline at end of file
+    @pyqtProperty(str)
+    def value(self):
+        return self._value
+
